@@ -44,16 +44,22 @@
 #include <utils.h>
 #include <list.h>
 #include <debug.h>
+#include "load_dir.h"
 #include "config/datadir.h"
 
 #include <stdlib.h>
-#include <curses.h>
 #include <ctype.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <dirent.h>
 #include <pwd.h>
 
+#if defined(__sun__)
+#include <ncurses.h>
+#else
+#include <curses.h>
+#endif
+
 static struct history cmd_history;
 static char *cmd_history_filename;
 static char *history_search_text = NULL;
@@ -442,14 +448,14 @@ static int get_number(char *str, char **end)
 
 static void cmd_seek(char *arg)
 {
-	int seek_mode = SEEK_SET;
+	int relative = 0;
 	int seek = 0, sign = 1, count;
 
 	switch (*arg) {
 	case '-':
 		sign = -1;
 	case '+':
-		seek_mode = SEEK_CUR;
+		relative = 1;
 		arg++;
 		break;
 	}
@@ -489,7 +495,7 @@ inside:
 	}
 
 	if (!*arg) {
-		player_seek(seek, seek_mode);
+		player_seek(seek, relative);
 		return;
 	}
 err:
@@ -973,8 +979,6 @@ static void cmd_run(char *arg)
 	}
 	argv[argc] = NULL;
 
-	free(av);
-
 	for (i = 0; argv[i]; i++)
 		d_print("ARG: '%s'\n", argv[i]);
 
@@ -1004,7 +1008,8 @@ static void cmd_run(char *arg)
 			case TREE_VIEW:
 			case SORTED_VIEW:
 				/* this must be done before sel_tis are unreffed */
-				free_str_array(argv);
+				free_str_array(av);
+				free(argv);
 
 				/* remove non-existed files, update tags for changed files */
 				cmus_update_tis(sel_tis, sel_tis_nr);
@@ -1014,7 +1019,8 @@ static void cmd_run(char *arg)
 			}
 		}
 	}
-	free_str_array(argv);
+	free_str_array(av);
+	free(argv);
 	for (i = 0; sel_tis[i]; i++)
 		track_info_unref(sel_tis[i]);
 	free(sel_tis);
@@ -1741,8 +1747,7 @@ static void expand_bind_args(const char *str)
 	 */
 	/* start and end pointers for context, key and function */
 	const char *cs, *ce, *ks, *ke, *fs;
-	char *tmp, **tails;
-	int i, c, k, len, pos, alloc, count;
+	int i, c, k, count;
 	int flag = parse_flags((const char **)&str, "f");
 	const char *force = "";
 
@@ -1774,34 +1779,34 @@ static void expand_bind_args(const char *str)
 	ke = strchr(ks, ' ');
 	if (ke == NULL) {
 		/* expand key */
-		len = strlen(ks);
-		tails = NULL;
-		alloc = 0;
-		pos = 0;
+		int len = strlen(ks);
+		PTR_ARRAY(array);
+
 		for (i = 0; key_table[i].name; i++) {
 			int cmp = strncmp(ks, key_table[i].name, len);
 			if (cmp > 0)
 				continue;
 			if (cmp < 0)
 				break;
-			tails = str_array_add(tails, &alloc, &pos, xstrdup(key_table[i].name + len));
+			ptr_array_add(&array, xstrdup(key_table[i].name + len));
 		}
 
-		if (pos == 0) {
+		if (!array.count)
 			return;
-		}
-		if (pos == 1) {
-			tmp = xstrjoin(tails[0], " ");
-			free(tails[0]);
-			tails[0] = tmp;
+
+		if (array.count == 1) {
+			char **ptrs = array.ptrs;
+			char *tmp = xstrjoin(ptrs[0], " ");
+			free(ptrs[0]);
+			ptrs[0] = tmp;
 		}
 
 		snprintf(expbuf, sizeof(expbuf), "%s%s %s", force, key_context_names[c], ks);
 
-		tails[pos] = NULL;
+		ptr_array_plug(&array);
 		tabexp.head = xstrdup(expbuf);
-		tabexp.tails = tails;
-		tabexp.nr_tails = pos;
+		tabexp.tails = array.ptrs;
+		tabexp.nr_tails = array.count;
 		return;
 	}
 
@@ -1857,9 +1862,9 @@ static void expand_unbind_args(const char *str)
 	/* :unbind context key */
 	/* start and end pointers for context and key */
 	const char *cs, *ce, *ks;
-	char **tails;
-	int c, len, pos, alloc;
 	const struct binding *b;
+	PTR_ARRAY(array);
+	int c, len;
 
 	cs = str;
 	ce = strchr(cs, ' ');
@@ -1881,33 +1886,30 @@ static void expand_unbind_args(const char *str)
 
 	/* expand key */
 	len = strlen(ks);
-	tails = NULL;
-	alloc = 0;
-	pos = 0;
 	b = key_bindings[c];
 	while (b) {
-		if (strncmp(ks, b->key->name, len) == 0)
-			tails = str_array_add(tails, &alloc, &pos, xstrdup(b->key->name + len));
+		if (!strncmp(ks, b->key->name, len))
+			ptr_array_add(&array, xstrdup(b->key->name + len));
 		b = b->next;
 	}
-	if (pos == 0)
+	if (!array.count)
 		return;
 
 	snprintf(expbuf, sizeof(expbuf), "%s %s", key_context_names[c], ks);
 
-	tails[pos] = NULL;
+	ptr_array_plug(&array);
 	tabexp.head = xstrdup(expbuf);
-	tabexp.tails = tails;
-	tabexp.nr_tails = pos;
+	tabexp.tails = array.ptrs;
+	tabexp.nr_tails = array.count;
 }
 
 static void expand_factivate(const char *str)
 {
 	/* "name1 name2 name3", expand only name3 */
 	struct filter_entry *e;
-	int str_len, len, i, pos, alloc;
 	const char *name;
-	char **tails;
+	PTR_ARRAY(array);
+	int str_len, len, i;
 
 	str_len = strlen(str);
 	i = str_len;
@@ -1919,20 +1921,17 @@ static void expand_factivate(const char *str)
 	len = str_len - i;
 	name = str + i;
 
-	tails = NULL;
-	alloc = 0;
-	pos = 0;
 	list_for_each_entry(e, &filters_head, node) {
-		if (strncmp(name, e->name, len) == 0)
-			tails = str_array_add(tails, &alloc, &pos, xstrdup(e->name + len));
+		if (!strncmp(name, e->name, len))
+			ptr_array_add(&array, xstrdup(e->name + len));
 	}
-	if (pos == 0)
+	if (!array.count)
 		return;
 
-	tails[pos] = NULL;
+	ptr_array_plug(&array);
 	tabexp.head = xstrdup(str);
-	tabexp.tails = tails;
-	tabexp.nr_tails = pos;
+	tabexp.tails = array.ptrs;
+	tabexp.nr_tails = array.count;
 }
 
 static void expand_options(const char *str)
@@ -1951,13 +1950,13 @@ static void expand_options(const char *str)
 			if (strcmp(var, opt->name) == 0) {
 				char buf[OPTION_MAX_SIZE];
 
-				tails = xnew(char *, 1);
+				tails = xnew(char *, 2);
 
 				buf[0] = 0;
 				opt->get(opt->id, buf);
 				tails[0] = xstrdup(buf);
-
 				tails[1] = NULL;
+
 				tabexp.head = xstrdup(str);
 				tabexp.tails = tails;
 				tabexp.nr_tails = 1;
@@ -2020,60 +2019,48 @@ static void expand_toptions(const char *str)
 	}
 }
 
-static void load_themes(const char *dir, const char *str, int *allocp)
+static void load_themes(const char *dirname, const char *str, struct ptr_array *array)
 {
-	DIR *d;
-	struct dirent *dirent;
+	struct directory dir;
+	const char *name, *dot;
 	int len = strlen(str);
 
-	d = opendir(dir);
-	if (d == NULL)
+	if (dir_open(&dir, dirname))
 		return;
 
-	while ((dirent = readdir(d)) != NULL) {
-		const char *dot, *name = dirent->d_name;
-		char filename[512];
-		struct stat s;
-
+	while ((name = dir_read(&dir))) {
+		if (!S_ISREG(dir.st.st_mode))
+			continue;
 		if (strncmp(name, str, len))
 			continue;
-
 		dot = strrchr(name, '.');
 		if (dot == NULL || strcmp(dot, ".theme"))
 			continue;
-
-		if (dot - name < len) {
+		if (dot - name < len)
 			/* str is  "foo.th"
 			 * matches "foo.theme"
 			 * which also ends with ".theme"
 			 */
 			continue;
-		}
-
-		snprintf(filename, sizeof(filename), "%s/%s", dir, name);
-		if (stat(filename, &s) || !S_ISREG(s.st_mode))
-			continue;
-
-		tabexp.tails = str_array_add(tabexp.tails, allocp, &tabexp.nr_tails,
-				xstrndup(name + len, dot - name - len));
+		ptr_array_add(array, xstrndup(name + len, dot - name - len));
 	}
-	closedir(d);
+	dir_close(&dir);
 }
 
 static void expand_colorscheme(const char *str)
 {
-	int alloc = 0;
+	PTR_ARRAY(array);
 
-	tabexp.nr_tails = 0;
+	load_themes(cmus_config_dir, str, &array);
+	load_themes(DATADIR "/cmus", str, &array);
 
-	load_themes(cmus_config_dir, str, &alloc);
-	load_themes(DATADIR "/cmus", str, &alloc);
+	if (array.count) {
+		ptr_array_sort(&array, strptrcmp);
 
-	if (alloc) {
-		qsort(tabexp.tails, tabexp.nr_tails, sizeof(char *), strptrcmp);
-
-		tabexp.tails[tabexp.nr_tails] = NULL;
+		ptr_array_plug(&array);
 		tabexp.head = xstrdup(str);
+		tabexp.tails = array.ptrs;
+		tabexp.nr_tails = array.count;
 	}
 }
 
