@@ -7,6 +7,10 @@
 
 #include <sf.h>
 
+extern int soft_vol;
+extern int soft_vol_l;
+extern int soft_vol_r;
+
 void op_load_plugins(void);
 void op_exit_plugins(void);
 
@@ -25,16 +29,6 @@ int op_select_any(void);
  */
 int op_open(sample_format_t sf);
 
-/*
- * returns:
- *     0 if sample format didn't change
- *     1 if sample format did change
- *     OP_ERROR_{} on error
- */
-int op_set_sf(sample_format_t sf);
-
-int op_second_size(void);
-
 /*
  * drop pcm data
  *
@@ -75,6 +69,8 @@ int op_reset(void);
 int op_set_volume(int left, int right);
 int op_get_volume(int *left, int *right);
 
+void op_set_soft_vol(int soft);
+
 /*
  * errors: OP_ERROR_{NO_PLUGIN, NOT_INITIALIZED, NOT_OPTION}
  */
