@@ -85,6 +85,9 @@ static int consumer_running = 1;
 static enum consumer_status consumer_status = CS_STOPPED;
 static int consumer_pos = 0;
 
+/* usually same as consumer_pos, sometimes less than consumer_pos */
+static int soft_vol_pos;
+
 /* locking {{{ */
 
 #define producer_lock() cmus_mutex_lock(&producer_mutex)
@@ -111,6 +114,7 @@ static void reset_buffer(void)
 {
 	buffer_reset();
 	consumer_pos = 0;
+	soft_vol_pos = 0;
 }
 
 static void set_buffer_sf(sample_format_t sf)
@@ -124,6 +128,77 @@ static void set_buffer_sf(sample_format_t sf)
 	}
 }
 
+#define SOFT_VOL_SCALE 65536
+
+/* coefficients for volumes 0..99, for 100 65536 is used
+ * data copied from alsa-lib src/pcm/pcm_softvol.c
+ */
+static const unsigned short soft_vol_db[100] = {
+	0x0000, 0x0110, 0x011c, 0x012f, 0x013d, 0x0152, 0x0161, 0x0179,
+	0x018a, 0x01a5, 0x01c1, 0x01d5, 0x01f5, 0x020b, 0x022e, 0x0247,
+	0x026e, 0x028a, 0x02b6, 0x02d5, 0x0306, 0x033a, 0x035f, 0x0399,
+	0x03c2, 0x0403, 0x0431, 0x0479, 0x04ac, 0x04fd, 0x0553, 0x058f,
+	0x05ef, 0x0633, 0x069e, 0x06ea, 0x0761, 0x07b5, 0x083a, 0x0898,
+	0x092c, 0x09cb, 0x0a3a, 0x0aeb, 0x0b67, 0x0c2c, 0x0cb6, 0x0d92,
+	0x0e2d, 0x0f21, 0x1027, 0x10de, 0x1202, 0x12cf, 0x1414, 0x14f8,
+	0x1662, 0x1761, 0x18f5, 0x1a11, 0x1bd3, 0x1db4, 0x1f06, 0x211d,
+	0x2297, 0x24ec, 0x2690, 0x292a, 0x2aff, 0x2de5, 0x30fe, 0x332b,
+	0x369f, 0x390d, 0x3ce6, 0x3f9b, 0x43e6, 0x46eb, 0x4bb3, 0x4f11,
+	0x5466, 0x5a18, 0x5e19, 0x6472, 0x68ea, 0x6ffd, 0x74f8, 0x7cdc,
+	0x826a, 0x8b35, 0x9499, 0x9b35, 0xa5ad, 0xad0b, 0xb8b7, 0xc0ee,
+	0xcdf1, 0xd71a, 0xe59c, 0xefd3
+};
+
+static inline void scale_sample(signed short *buf, int i, int vol)
+{
+	int sample = buf[i];
+
+	if (sample < 0) {
+		buf[i] = (sample * vol - SOFT_VOL_SCALE / 2) / SOFT_VOL_SCALE;
+	} else {
+		buf[i] = (sample * vol + SOFT_VOL_SCALE / 2) / SOFT_VOL_SCALE;
+	}
+}
+
+static void soft_vol_scale(char *buffer, int *countp)
+{
+	signed short *buf;
+	int count = *countp;
+	int ch, bits, l, r, i;
+
+	if (consumer_pos != soft_vol_pos) {
+		int offs = soft_vol_pos - consumer_pos;
+
+		BUG_ON(offs < 0);
+		if (offs >= count)
+			return;
+		buffer += offs;
+		count -= offs;
+	}
+	soft_vol_pos += count;
+	buf = (signed short *)buffer;
+
+	if (soft_vol_l == 100 && soft_vol_r == 100)
+		return;
+
+	ch = sf_get_channels(buffer_sf);
+	bits = sf_get_bits(buffer_sf);
+	if (ch != 2 || bits != 16)
+		return;
+
+	l = SOFT_VOL_SCALE;
+	r = SOFT_VOL_SCALE;
+	if (soft_vol_l != 100)
+		l = soft_vol_db[soft_vol_l];
+	if (soft_vol_r != 100)
+		r = soft_vol_db[soft_vol_r];
+
+	for (i = 0; i < count / 4; i++) {
+		scale_sample(buf, i * 2, l);
+		scale_sample(buf, i * 2 + 1, r);
+	}
+}
+
 static inline int buffer_second_size(void)
 {
 	return sf_get_second_size(buffer_sf);
@@ -225,19 +300,23 @@ static void __FORMAT(2, 3) player_op_error(int rc, const char *format, ...)
 /* FIXME: don't poll */
 static void mixer_check(void)
 {
-	static struct timeval old_st = { 0L, 0L };
-	struct timeval st;
+	static struct timeval old_t = { 0L, 0L };
+	struct timeval t;
+	long usec, sec;
 	int l, r;
 
-	gettimeofday(&st, NULL);
-	if (st.tv_sec == old_st.tv_sec) {
-		unsigned long usecs = st.tv_sec - old_st.tv_sec;
-
-		if (usecs < 300e6)
-			return;
+	gettimeofday(&t, NULL);
+	usec = t.tv_usec - old_t.tv_usec;
+	sec = t.tv_sec - old_t.tv_sec;
+	if (sec) {
+		/* multiplying sec with 1e6 can overflow */
+		usec += 1e6L;
 	}
-	old_st = st;
-	if (op_get_volume(&l, &r) == 0)
+	if (usec < 300e3)
+		return;
+
+	old_t = t;
+	if (!op_get_volume(&l, &r))
 		volume_update(l, r);
 }
 
@@ -371,6 +450,7 @@ static void __producer_play(void)
 				player_ip_error(rc, "opening file `%s'", filename);
 				ip_delete(ip);
 			} else {
+				ip_setup(ip);
 				producer_status = PS_PLAYING;
 			}
 			free(filename);
@@ -389,6 +469,7 @@ static void __producer_play(void)
 			ip_delete(ip);
 			producer_status = PS_UNLOADED;
 		} else {
+			ip_setup(ip);
 			producer_status = PS_PLAYING;
 		}
 	} else if (producer_status == PS_PAUSED) {
@@ -485,6 +566,33 @@ static void __consumer_pause(void)
 
 /* setting consumer status }}} */
 
+static int change_sf(sample_format_t sf, int drop)
+{
+	int old_sf = buffer_sf;
+
+	set_buffer_sf(sf);
+	if (buffer_sf != old_sf) {
+		/* reopen */
+		int rc;
+
+		if (drop)
+			op_drop();
+		op_close();
+		rc = op_open(buffer_sf);
+		if (rc) {
+			player_op_error(rc, "opening audio device");
+			consumer_status = CS_STOPPED;
+			__producer_stop();
+			return rc;
+		}
+	} else if (consumer_status == CS_PAUSED) {
+		op_drop();
+		op_unpause();
+	}
+	consumer_status = CS_PLAYING;
+	return 0;
+}
+
 static void __consumer_handle_eof(void)
 {
 	char *filename;
@@ -497,8 +605,6 @@ static void __consumer_handle_eof(void)
 	}
 
 	if (get_next(&filename) == 0) {
-		int rc;
-
 		__producer_unload();
 		ip = ip_new(filename);
 		producer_status = PS_STOPPED;
@@ -510,17 +616,9 @@ static void __consumer_handle_eof(void)
 				file_changed();
 			} else {
 				/* PS_PLAYING */
-				set_buffer_sf(ip_get_sf(ip));
-				rc = op_set_sf(buffer_sf);
-				if (rc < 0) {
-					__producer_stop();
-					consumer_status = CS_STOPPED;
-					player_op_error(rc, "setting sample format");
-					file_changed();
-				} else {
-					file_changed();
+				file_changed();
+				if (!change_sf(ip_get_sf(ip), 0))
 					__prebuffer();
-				}
 			}
 		} else {
 			__consumer_drain_and_stop();
@@ -605,6 +703,8 @@ static void *consumer_loop(void *arg)
 			}
 			if (size > space)
 				size = space;
+			if (soft_vol)
+				soft_vol_scale(rpos, &size);
 			rc = op_write(rpos, size);
 			if (rc < 0) {
 				d_print("op_write returned %d %s\n", rc,
@@ -817,8 +917,6 @@ void player_pause(void)
 
 void player_set_file(const char *filename)
 {
-	int rc;
-
 	player_lock();
 	__producer_set_file(filename);
 	if (producer_status == PS_UNLOADED) {
@@ -833,30 +931,7 @@ void player_set_file(const char *filename)
 			__consumer_stop();
 			goto out;
 		}
-
-		/* must do op_drop() here because op_set_sf()
-		 * might call op_close() which calls drain()
-		 */
-		op_drop();
-
-		set_buffer_sf(ip_get_sf(ip));
-		rc = op_set_sf(buffer_sf);
-		if (rc == 0) {
-			/* device wasn't reopened */
-			if (consumer_status == CS_PAUSED) {
-				/* status was paused => need to unpause */
-				rc = op_unpause();
-				d_print("op_unpause: %d\n", rc);
-			}
-			consumer_status = CS_PLAYING;
-		} else if (rc == 1) {
-			/* device was reopened */
-			consumer_status = CS_PLAYING;
-		} else {
-			__producer_stop();
-			player_op_error(rc, "setting sample format");
-			consumer_status = CS_STOPPED;
-		}
+		change_sf(ip_get_sf(ip), 1);
 	}
 out:
 	__player_status_changed();
@@ -867,8 +942,6 @@ out:
 
 void player_play_file(const char *filename)
 {
-	int rc;
-
 	player_lock();
 	__producer_set_file(filename);
 	if (producer_status == PS_UNLOADED) {
@@ -891,30 +964,7 @@ void player_play_file(const char *filename)
 		if (consumer_status == CS_STOPPED)
 			__producer_stop();
 	} else {
-		/* PS_PLAYING, CS_PLAYING,CS_PAUSED */
-		/* must do op_drop() here because op_set_sf()
-		 * might call op_close() which calls drain()
-		 */
-		op_drop();
-
-		set_buffer_sf(ip_get_sf(ip));
-		rc = op_set_sf(buffer_sf);
-		if (rc == 0) {
-			/* device wasn't reopened */
-			if (consumer_status == CS_PAUSED) {
-				/* status was paused => need to unpause */
-				rc = op_unpause();
-				d_print("op_unpause: %d\n", rc);
-			}
-			consumer_status = CS_PLAYING;
-		} else if (rc == 1) {
-			/* device was reopened */
-			consumer_status = CS_PLAYING;
-		} else {
-			__producer_stop();
-			player_op_error(rc, "setting sample format");
-			consumer_status = CS_STOPPED;
-		}
+		change_sf(ip_get_sf(ip), 1);
 	}
 out:
 	__player_status_changed();
@@ -923,7 +973,7 @@ out:
 	player_unlock();
 }
 
-void player_seek(double offset, int whence)
+void player_seek(double offset, int relative)
 {
 	player_lock();
 	if (consumer_status == CS_PLAYING || consumer_status == CS_PAUSED) {
@@ -938,8 +988,7 @@ void player_seek(double offset, int whence)
 			player_unlock();
 			return;
 		}
-		if (whence == SEEK_CUR) {
-			/* relative to current position */
+		if (relative) {
 			new_pos = pos + offset;
 			if (new_pos < 0.0)
 				new_pos = 0.0;
@@ -957,7 +1006,6 @@ void player_seek(double offset, int whence)
 				}
 			}
 		} else {
-			/* absolute position */
 			new_pos = offset;
 			if (new_pos < 0.0) {
 				d_print("seek offset negative\n");
@@ -977,6 +1025,7 @@ void player_seek(double offset, int whence)
 			op_drop();
 			reset_buffer();
 			consumer_pos = new_pos * buffer_second_size();
+			soft_vol_pos = consumer_pos;
 			__consumer_position_update();
 		} else {
 			d_print("error: ip_seek returned %d\n", rc);
@@ -988,7 +1037,7 @@ void player_seek(double offset, int whence)
 /*
  * change output plugin without stopping playback
  */
-int player_set_op(const char *name)
+void player_set_op(const char *name)
 {
 	int rc, l, r;
 
@@ -1015,7 +1064,7 @@ int player_set_op(const char *name)
 		__producer_stop();
 		player_op_error(rc, "selecting output plugin '%s'", name);
 		player_unlock();
-		return rc;
+		return;
 	}
 
 	if (consumer_status == CS_PLAYING || consumer_status == CS_PAUSED) {
@@ -1026,7 +1075,7 @@ int player_set_op(const char *name)
 			__producer_stop();
 			player_op_error(rc, "opening audio device");
 			player_unlock();
-			return rc;
+			return;
 		}
 		if (consumer_status == CS_PAUSED)
 			op_pause();
@@ -1036,8 +1085,6 @@ int player_set_op(const char *name)
 		volume_update(l, r);
 
 	player_unlock();
-	d_print("rc = %d\n", rc);
-	return rc;
 }
 
 char *player_get_op(void)
@@ -1114,12 +1161,26 @@ int player_set_volume(int left, int right)
 
 	consumer_lock();
 	rc = op_set_volume(left, right);
-	consumer_unlock();
-	if (rc == 0)
+	if (!rc)
 		volume_update(left, right);
+	consumer_unlock();
 	return rc;
 }
 
+void player_set_soft_vol(int soft)
+{
+	int l, r;
+
+	consumer_lock();
+	/* don't mess with soft_vol_pos if soft_vol is already true */
+	if (!soft_vol)
+		soft_vol_pos = consumer_pos;
+	op_set_soft_vol(soft);
+	if (!op_get_volume(&l, &r))
+		volume_update(l, r);
+	consumer_unlock();
+}
+
 int player_set_op_option(unsigned int id, const char *val)
 {
 	int rc;
