@@ -4,6 +4,24 @@
 #
 # This file is licensed under the GPLv2.
 
+# C compiler
+# ----------
+# CC          default gcc
+# LD          default $CC
+# LDFLAGS     common linker flags for CC
+#
+# C++ Compiler
+# ------------
+# CXX         default g++
+# CXXLD       default $CXX
+# CXXLDFLAGS  common linker flags for CXX
+#
+# Common for C and C++
+# --------------------
+# SOFLAGS     flags for compiling position independent code (-fPIC)
+# LDSOFLAGS   flags for linking shared libraries
+# LDDLFLAGS   flags for linking dynamically loadable modules
+
 msg_checking()
 {
 	printf "checking $@... "
@@ -27,7 +45,7 @@ check_program()
 {
 	argc check_program $# 1 2
 	msg_checking "for program $1"
-	__cp_file=$(path_find "$1")
+	__cp_file=`path_find "$1"`
 	if test $? -eq 0
 	then
 		msg_result $__cp_file
@@ -53,7 +71,8 @@ cxx_supports()
 
 # @flag: option flag(s) to check
 #
-# add @flag to CFLAGS if CC accepts it
+# add @flag to EXTRA_CFLAGS if CC accepts it
+# EXTRA_CFLAGS are added to CFLAGS in the end of configuration
 check_cc_flag()
 {
 	argc check_cc_flag $# 1
@@ -62,7 +81,7 @@ check_cc_flag()
 	msg_checking "for CFLAGS $*"
 	if cc_supports $*
 	then
-		CFLAGS="$CFLAGS $*"
+		EXTRA_CFLAGS="$EXTRA_CFLAGS $*"
 		msg_result "yes"
 		return 0
 	else
@@ -73,7 +92,8 @@ check_cc_flag()
 
 # @flag: option flag(s) to check
 #
-# add @flag to CXXFLAGS if CXX accepts it
+# add @flag to EXTRA_CXXFLAGS if CXX accepts it
+# EXTRA_CXXFLAGS are added to CXXFLAGS in the end of configuration
 check_cxx_flag()
 {
 	argc check_cxx_flag $# 1
@@ -82,7 +102,7 @@ check_cxx_flag()
 	msg_checking "for CXXFLAGS $*"
 	if cxx_supports $*
 	then
-		CXXFLAGS="$CXXFLAGS $*"
+		EXTRA_CXXFLAGS="$EXTRA_CXXFLAGS $*"
 		msg_result "yes"
 		return 0
 	else
@@ -91,19 +111,37 @@ check_cxx_flag()
 	fi
 }
 
-# extra flags for linking shared libraries and dynamically loadable modules
-# called by check_cc
-check_shared_flags()
+cc_cxx_common()
 {
-	# most of this is from Autoconf
-	# the OS X specific parts are likely wrong
-	case $(uname -s) in
-	rhapsody* | darwin1.[012])
-		LDSOFLAGS="-dynamic"
-		LDDLFLAGS="-bundle -undefined suppress"
+	test "$cc_cxx_common_done" && return 0
+	cc_cxx_common_done=yes
+
+	var_default SOFLAGS "-fPIC"
+	var_default LDSOFLAGS "-shared"
+	var_default LDDLFLAGS "-shared"
+
+	common_cf=
+	common_lf=
+	COMPAT_LIBS=
+
+	case `uname -s` in
+	*BSD)
+		common_cf="$common_cf -I/usr/local/include"
+		common_lf="$common_lf -L/usr/local/lib"
 		;;
-	darwin*)
-		# Darwin 1.3
+	Darwin)
+		# fink
+		if test -d /sw/lib
+		then
+			common_cf="$common_cf -I/sw/include"
+			common_lf="$common_lf -L/sw/lib"
+		fi
+		# darwinports
+		if test -d /opt/local/lib
+		then
+			common_cf="$common_cf -I/opt/local/include"
+			common_lf="$common_lf -L/opt/local/lib"
+		fi
 		LDSOFLAGS="-dynamic"
 		case ${MACOSX_DEPLOYMENT_TARGET} in
 		10.[012])
@@ -117,57 +155,66 @@ check_shared_flags()
 			;;
 		esac
 		;;
-	*)
-		LDSOFLAGS="-shared"
-		LDDLFLAGS="-shared"
+	SunOS)
+		common_cf="$common_cf -D__EXTENSIONS__ -I/usr/local/include"
+		common_lf="$common_lf -R/usr/local/lib -L/usr/local/lib"
+
+		# this is ugly but can be removed after Solaris is either
+		# dead or fixed
+
+		# connect() etc.
+		try_link -lsocket && COMPAT_LIBS="$COMPAT_LIBS -lsocket"
+
+		# gethostbyname()
+		try_link -lnsl && COMPAT_LIBS="$COMPAT_LIBS -lnsl"
+
+		# nanosleep()
+		if try_link -lrt
+		then
+			COMPAT_LIBS="$COMPAT_LIBS -lrt"
+		elif try_link -lposix4
+		then
+			COMPAT_LIBS="$COMPAT_LIBS -lposix4"
+		fi
 		;;
 	esac
-	makefile_vars LDSOFLAGS LDDLFLAGS
+	makefile_vars SOFLAGS LDSOFLAGS LDDLFLAGS COMPAT_LIBS
 }
 
-# adds CC, LD, CFLAGS, LDFLAGS and SOFLAGS to config.mk
+# CC, LD, CFLAGS, LDFLAGS, SOFLAGS, LDSOFLAGS, LDDLFLAGS
 check_cc()
 {
 	var_default CC ${CROSS}gcc
 	var_default LD $CC
-	var_default CFLAGS "-O2"
+	var_default CFLAGS "-g -O2 -Wall"
 	var_default LDFLAGS ""
-	var_default SOFLAGS "-fPIC"
-	if check_program $CC
-	then
-		case $(uname -s) in
-			*BSD)
-				CFLAGS="$CFLAGS -I/usr/local/include"
-				LDFLAGS="$LDFLAGS -L/usr/local/lib"
-				;;
-		esac
-		makefile_vars CC LD CFLAGS LDFLAGS SOFLAGS
-		check_shared_flags
-		return 0
-	fi
-	return 1
+	check_program $CC || return 1
+
+	cc_cxx_common
+	CFLAGS="$CFLAGS $common_cf"
+	LDFLAGS="$LDFLAGS $common_lf"
+
+	makefile_vars CC LD CFLAGS LDFLAGS
+	__check_lang=c
+	return 0
 }
 
-# adds CXX, CXXLD, CXXFLAGS and CXXLDFLAGS to config.mk
+# CXX, CXXLD, CXXFLAGS, CXXLDFLAGS, SOFLAGS, LDSOFLAGS, LDDLFLAGS
 check_cxx()
 {
 	var_default CXX ${CROSS}g++
 	var_default CXXLD $CXX
-	var_default CXXFLAGS "-O2"
+	var_default CXXFLAGS "-g -O2 -Wall"
 	var_default CXXLDFLAGS ""
-	if check_program $CXX
-	then
-		case $(uname -s) in
-			*BSD)
-				CXXFLAGS="$CXXFLAGS -I/usr/local/include"
-				CXXLDFLAGS="$CXXLDFLAGS -L/usr/local/lib"
-				;;
-		esac
-		makefile_vars CXX CXXLD CXXFLAGS CXXLDFLAGS
-		check_shared_flags
-		return 0
-	fi
-	return 1
+	check_program $CXX || return 1
+
+	cc_cxx_common
+	CXXFLAGS="$CXXFLAGS $common_cf"
+	CXXLDFLAGS="$CXXLDFLAGS $common_lf"
+
+	makefile_vars CXX CXXLD CXXFLAGS CXXLDFLAGS
+	__check_lang=cxx
+	return 0
 }
 
 # check if CC can generate dependencies (.dep-*.o files)
@@ -177,7 +224,7 @@ check_cc_depgen()
 	msg_checking "if CC can generate dependency information"
 	if cc_supports -MMD -MP -MF /dev/null
 	then
-		CFLAGS="$CFLAGS -MMD -MP -MF .dep-\$(subst /,-,\$@)"
+		EXTRA_CFLAGS="$EXTRA_CFLAGS -MMD -MP -MF .dep-\$(subst /,-,\$@)"
 		msg_result yes
 	else
 		msg_result no
@@ -192,7 +239,7 @@ check_cxx_depgen()
 	msg_checking "if CXX can generate dependency information"
 	if cxx_supports -MMD -MP -MF /dev/null
 	then
-		CXXFLAGS="$CXXFLAGS -MMD -MP -MF .dep-\$(subst /,-,\$@)"
+		EXTRA_CXXFLAGS="$EXTRA_CXXFLAGS -MMD -MP -MF .dep-\$(subst /,-,\$@)"
 		msg_result yes
 	else
 		msg_result no
@@ -253,7 +300,7 @@ check_library()
 {
 	argc check_library $# 3 3
 	msg_checking "for ${1}_LIBS ($3)"
-	if try_link "$3" >/dev/null 2>&1
+	if try_link $3
 	then
 		msg_result yes
 		makefile_var ${1}_CFLAGS "$2"
@@ -267,7 +314,7 @@ check_library()
 
 # run pkg-config
 #
-# @name:    variable prefix (e.g. GLIB -> GLIB_CFLAGS, GLIB_LIBS)
+# @prefix:  variable prefix (e.g. GLIB -> GLIB_CFLAGS, GLIB_LIBS)
 # @modules: the argument for pkg-config
 # @cflags:  CFLAGS to use if pkg-config failed (optional)
 # @libs:    LIBS to use if pkg-config failed (optional)
@@ -278,11 +325,12 @@ check_library()
 #   ---
 #   check_glib()
 #   {
-#     pkg_config GLIB "glib-2.0 >= 2.2"
-#     return $?
+#           pkg_config GLIB "glib-2.0 >= 2.2"
+#           return $?
 #   }
 #
-#   run_checks check_cc check_glib
+#   check check_cc
+#   check check_glib
 #   ---
 #   GLIB_CFLAGS and GLIB_LIBS are automatically added to Makefile
 pkg_config()
@@ -298,11 +346,11 @@ pkg_config()
 	if test "$PKG_CONFIG" != "no" && $PKG_CONFIG --exists "$2" >/dev/null 2>&1
 	then
 		# pkg-config is installed and the .pc file exists
-		__pc_libs="$($PKG_CONFIG --libs ""$2"")"
+		__pc_libs="`$PKG_CONFIG --libs ""$2""`"
 		msg_result "$__pc_libs"
 
 		msg_checking "for ${1}_CFLAGS (pkg-config)"
-		__pc_cflags="$($PKG_CONFIG --cflags ""$2"")"
+		__pc_cflags="`$PKG_CONFIG --cflags ""$2""`"
 		msg_result "$__pc_cflags"
 
 		makefile_var ${1}_CFLAGS "$__pc_cflags"
@@ -336,33 +384,34 @@ pkg_check_modules()
 	pkg_config "$@"
 }
 
-# run <name>-config
+# run *-config
 #
-# @name:    name
+# @prefix:  variable prefix (e.g. ARTS -> ARTS_CFLAGS, ARTS_LIBS)
 # @program: the -config program
 #
 # example:
 #   ---
-#   check_cppunit()
+#   check_arts()
 #   {
-#     app_config CPPUNIT cppunit-config
-#     return $?
+#           app_config ARTS artsc-config
+#           return $?
 #   }
 #
-#   run_checks check_cc check_glib
+#   check check_cc
+#   check check_arts
 #   ---
-#   CPPUNIT_CFLAGS and CPPUNIT_LIBS are automatically added to config.mk
+#   ARTS_CFLAGS and ARTS_LIBS are automatically added to config.mk
 app_config()
 {
 	argc app_config $# 2 2
 	check_program $2 || return 1
 
 	msg_checking "for ${1}_CFLAGS"
-	__ac_cflags="$($2 --cflags)"
+	__ac_cflags="`$2 --cflags`"
 	msg_result "$__ac_cflags"
 
 	msg_checking "for ${1}_LIBS"
-	__ac_libs="$($2 --libs)"
+	__ac_libs="`$2 --libs`"
 	msg_result "$__ac_libs"
 
 	makefile_var ${1}_CFLAGS "$__ac_cflags"
@@ -371,36 +420,38 @@ app_config()
 }
 
 # @contents:  file contents to compile
+# @cflags:    extra cflags (optional)
 try_compile()
 {
-	argc try_compile $# 1 1
-	__src=$(tmp_file prog.c)
-	__obj=$(tmp_file prog.o)
-	echo "$1" > $__src || exit 1
-	$CC -c $__src -o $__obj 2>/dev/null
+	argc try_compile $# 1
+	case $__check_lang in
+	c)
+		__src=`tmp_file prog.c`
+		__obj=`tmp_file prog.o`
+		echo "$1" > $__src || exit 1
+		shift
+		__cmd="$CC -c $CFLAGS $@ $__src -o $__obj"
+		$CC -c $CFLAGS "$@" $__src -o $__obj 2>/dev/null
+		;;
+	cxx)
+		__src=`tmp_file prog.cc`
+		__obj=`tmp_file prog.o`
+		echo "$1" > $__src || exit 1
+		shift
+		__cmd="$CXX -c $CXXFLAGS $@ $__src -o $__obj"
+		$CXX -c $CXXFLAGS "$@" $__src -o $__obj 2>/dev/null
+		;;
+	esac
 	return $?
 }
 
 # tries to link against a lib
 # 
-# @ldadd:  something like "-L/usr/X11R6/lib -lX11"
+# @ldadd:  something like -L/usr/X11R6/lib -lX11
 try_link()
 {
-	argc try_link $# 1 1
-
-	__src=$(tmp_file prog.c)
-	__obj=$(tmp_file prog.o)
-	__exe=$(tmp_file prog)
-
-	echo "
-int main(int argc, char *argv[])
-{
-	return 0;
-}
-" > $__src || exit 1
-
-	$CC -c $__src -o $__obj || return 1
-	$LD $LDFLAGS $1 -o $__exe $__obj
+	try_compile "int main(int argc, char *argv[]) { return 0; }" || __compile_failed
+	__try_link "$@"
 	return $?
 }
 
@@ -410,21 +461,13 @@ int main(int argc, char *argv[])
 check_endianness()
 {
 	msg_checking "byte order"
-	__src=$(tmp_file byteorder.c)
-	__obj=$(tmp_file byteorder.o)
-	__exe=$(tmp_file byteorder)
-
-	echo "
+	try_compile "
 int main(int argc, char *argv[])
 {
 	unsigned int i = 1;
-
 	return *(char *)&i;
-}
-" > $__src || exit 1
-
-	$CC -c $__src -o $__obj 2>/dev/null || exit 1
-	$LD -o $__exe $__obj 2>/dev/null || return 1
+}" || __compile_failed
+	__try_link || __link_failed
 	if ./$__exe
 	then
 		msg_result "big-endian"
@@ -436,24 +479,21 @@ int main(int argc, char *argv[])
 	return 0
 }
 
-# check if linking against @ldadd is possible
-# use check_library instead if possible
+# check if @header can be included
 #
-# @name:   user visible name
-# @ldadd:  arg passed to try_link
-check_lib()
+# @header
+# @cflags   -I/some/path (optional)
+check_header()
 {
-	argc check_lib $# 2 2
-	msg_checking "for $1"
-	try_link "$2" > /dev/null 2>&1
-	if test $? -eq 0
+	argc check_header $# 1
+	msg_checking "for header <$1>"
+	if try_compile "#include <$1>" "$@"
 	then
-		msg_result "yes"
+		msg_result yes
 		return 0
-	else
-		msg_result "no"
-		return 1
 	fi
+	msg_result no
+	return 1
 }
 
 # check X11 libs
@@ -489,13 +529,13 @@ check_pthread()
 # adds DL_LIBS to config.mk
 check_dl()
 {
-	for __libs in "-ldl -Wl,--export-dynamic" "-Wl,--export-dynamic" "-ldl"
+	for DL_LIBS in "-ldl -Wl,--export-dynamic" "-Wl,--export-dynamic" "-ldl"
 	do
-		check_library DL "" "$__libs" && return 0
+		check_library DL "" "$DL_LIBS" && return 0
 	done
 	echo "assuming -ldl is not needed"
-	makefile_var DL_LIBS ""
-	makefile_var DL_CFLAGS ""
+	DL_LIBS=
+	makefile_vars DL_LIBS DL_CFLAGS
 	return 0
 }
 
@@ -504,11 +544,10 @@ check_dl()
 # adds ICONV_CFLAGS and ICONV_LIBS to config.mk
 check_iconv()
 {
-	if ! check_library ICONV "" "-liconv"
-	then
-		echo "assuming libc contains iconv"
-		makefile_var ICONV_CFLAGS ""
-		makefile_var ICONV_LIBS ""
-	fi
+	check_library ICONV "" "-liconv" && return 0
+
+	echo "assuming libc contains iconv"
+	makefile_var ICONV_CFLAGS ""
+	makefile_var ICONV_LIBS ""
 	return 0
 }
