@@ -48,7 +48,6 @@
 #include <stdio.h>
 #include <errno.h>
 #include <sys/ioctl.h>
-#include <curses.h>
 #include <ctype.h>
 #include <dirent.h>
 #include <locale.h>
@@ -57,6 +56,14 @@
 #include <signal.h>
 #include <stdarg.h>
 
+#if defined(__sun__)
+/* TIOCGWINSZ */
+#include <termios.h>
+#include <ncurses.h>
+#else
+#include <curses.h>
+#endif
+
 /* defined in <term.h> but without const */
 char *tgetstr(const char *id, char **area);
 char *tgoto(const char *cap, int col, int row);
@@ -314,24 +321,22 @@ static void dump_print_buffer(int row, int col)
 	}
 }
 
-static void sprint(int row, int col, const char *str, int width, int indent)
+/* print @str into @buf
+ *
+ * if @str is shorter than @width pad with spaces
+ * if @str is wider than @width truncate and add "..."
+ */
+static int format_str(char *buf, const char *str, int width)
 {
-	int s, d, ellipsis_pos = 0, cut_double_width = 0;
+	int s = 0, d = 0, ellipsis_pos = 0, cut_double_width = 0;
 
-	width -= 2 + indent;
-	d = indent + 1;
-	memset(print_buffer, ' ', d);
-	s = 0;
 	while (1) {
 		uchar u;
 		int w;
 
-		if (width == 3)
-			ellipsis_pos = d;
-
 		u_get_char(str, &s, &u);
 		if (u == 0) {
-			memset(print_buffer + d, ' ', width);
+			memset(buf + d, ' ', width);
 			d += width;
 			break;
 		}
@@ -351,17 +356,26 @@ static void sprint(int row, int col, const char *str, int width, int indent)
 			d = ellipsis_pos;
 			if (cut_double_width) {
 				/* first half of the double-width char */
-				print_buffer[d - 1] = ' ';
+				buf[d - 1] = ' ';
 			}
-			print_buffer[d++] = '.';
-			print_buffer[d++] = '.';
-			print_buffer[d++] = '.';
+			buf[d++] = '.';
+			buf[d++] = '.';
+			buf[d++] = '.';
 			break;
 		}
-		u_set_char(print_buffer, &d, u);
+		u_set_char(buf, &d, u);
 	}
-	print_buffer[d++] = ' ';
-	print_buffer[d++] = 0;
+	return d;
+}
+
+static void sprint(int row, int col, const char *str, int width)
+{
+	int pos = 0;
+
+	print_buffer[pos++] = ' ';
+	pos += format_str(print_buffer + pos, str, width - 2);
+	print_buffer[pos++] = ' ';
+	print_buffer[pos] = 0;
 	dump_print_buffer(row, col);
 }
 
@@ -389,11 +403,11 @@ static void sprint_ascii(int row, int col, const char *str, int len)
 
 static void print_tree(struct window *win, int row, struct iter *iter)
 {
-	const char *noname = "<no name>";
+	const char *str;
 	struct artist *artist;
 	struct album *album;
 	struct iter sel;
-	int current, selected, active;
+	int current, selected, active, pos;
 
 	artist = iter_to_artist(iter);
 	album = iter_to_album(iter);
@@ -409,11 +423,21 @@ static void print_tree(struct window *win, int row, struct iter *iter)
 	selected = iters_equal(iter, &sel);
 	active = lib_cur_win == lib_tree_win;
 	bkgdset(pairs[(active << 2) | (selected << 1) | current]);
+
+	pos = 0;
+	print_buffer[pos++] = ' ';
+	str = artist->name;
 	if (album) {
-		sprint(tree_win_y + row + 1, tree_win_x, album->name ? : noname, tree_win_w, 2);
-	} else {
-		sprint(tree_win_y + row + 1, tree_win_x, artist->name ? : noname, tree_win_w, 0);
+		print_buffer[pos++] = ' ';
+		print_buffer[pos++] = ' ';
+		str = album->name;
 	}
+	if (!str)
+		str = "<no name>";
+	pos += format_str(print_buffer + pos, str, tree_win_w - pos - 1);
+	print_buffer[pos++] = ' ';
+	print_buffer[pos++] = 0;
+	dump_print_buffer(tree_win_y + row + 1, tree_win_x);
 }
 
 static inline void fopt_set_str(struct format_option *fopt, const char *str)
@@ -595,7 +619,7 @@ static void print_browser(struct window *win, int row, struct iter *iter)
 
 	/* file name encoding == terminal encoding. no need to convert */
 	if (using_utf8) {
-		sprint(row + 1, 0, e->name, COLS, 0);
+		sprint(row + 1, 0, e->name, COLS);
 	} else {
 		sprint_ascii(row + 1, 0, e->name, COLS);
 	}
@@ -611,14 +635,26 @@ static void print_filter(struct window *win, int row, struct iter *iter)
 	/* row selected? */
 	int selected;
 	/* is the filter currently active? */
-	int current = e->active;
+	int current = !!e->act_stat;
+	const char stat_chars[3] = " *!";
+	int ch1, ch2, ch3, pos;
 
 	window_get_sel(win, &sel);
 	selected = iters_equal(iter, &sel);
 	bkgdset(pairs[(active << 2) | (selected << 1) | current]);
 
-	snprintf(buf, sizeof(buf), "%c %-15s  %s", e->selected ? '*' : ' ', e->name, e->filter);
-	sprint(row + 1, 0, buf, COLS, 0);
+	ch1 = ' ';
+	ch3 = ' ';
+	if (e->sel_stat != e->act_stat) {
+		ch1 = '[';
+		ch3 = ']';
+	}
+	ch2 = stat_chars[e->sel_stat];
+	snprintf(buf, sizeof(buf), "%c%c%c%-15s  %s", ch1, ch2, ch3, e->name, e->filter);
+	pos = format_str(print_buffer, buf, COLS - 1);
+	print_buffer[pos++] = ' ';
+	print_buffer[pos] = 0;
+	dump_print_buffer(row + 1, 0);
 }
 
 static void update_window(struct window *win, int x, int y, int w, const char *title,
@@ -1010,7 +1046,7 @@ static void do_update_commandline(void)
 }
 
 /* lock player_info! */
-static const char *get_stream_title(const char *metadata)
+static const char *get_stream_title(void)
 {
 	static char stream_title[255 * 16 + 1];
 	char *ptr, *title;
@@ -1049,7 +1085,7 @@ static void do_update_titleline(void)
 
 		fill_track_fopts_track_info(cur_track_info);
 		if (is_url(cur_track_info->filename)) {
-			const char *title = get_stream_title(player_info.metadata);
+			const char *title = get_stream_title();
 
 			if (title == NULL)
 				use_alt_format = 1;
@@ -1427,7 +1463,7 @@ static void spawn_status_program(void)
 	player_info_lock();
 	status = player_info.status;
 	if (status == 1)
-		stream_title = get_stream_title(player_info.metadata);
+		stream_title = get_stream_title();
 	player_info_unlock();
 
 	i = 0;
@@ -1841,10 +1877,28 @@ static void init_curses(void)
 	if (!t_fs)
 		t_ts = NULL;
 
-	term = getenv("TERM");
-	if (!t_ts && term && !strcmp(term, "screen")) {
-		t_ts = "\033k";
-		t_fs = "\033\\";
+	if (!t_ts && (term = getenv("TERM"))) {
+		/*
+		 * Eterm:            Eterm
+		 * aterm:            rxvt
+		 * mlterm:           xterm
+		 * terminal (xfce):  xterm
+		 * urxvt:            rxvt-unicode
+		 * xterm:            xterm, xterm-{,16,88,256}color
+		 */
+		if (!strcmp(term, "screen")) {
+			t_ts = "\033k";
+			t_fs = "\033\\";
+		} else if (!strncmp(term, "xterm", 5) ||
+			   !strncmp(term, "rxvt", 4) ||
+			   !strcmp(term, "Eterm")) {
+			/* \033]1;  change icon
+			 * \033]2;  change title
+			 * \033]0;  change both
+			 */
+			t_ts = "\033]0;";
+			t_fs = "\007";
+		}
 	}
 }
 
@@ -1983,7 +2037,7 @@ int main(int argc, char *argv[])
 	}
 	misc_init();
 	if (server_address == NULL)
-		server_address = xstrjoin(home_dir, "/.cmus/socket");
+		server_address = xstrjoin(cmus_config_dir, "/socket");
 	debug_init();
 	d_print("charset = '%s'\n", charset);
 
